@@ -231,7 +231,7 @@ def rastrigin(individual):
        * - Global optima
          - :math:`x_i = 0, \\forall i \in \\lbrace 1 \\ldots N\\rbrace`, :math:`f(\mathbf{x}) = 0`
        * - Function
-         - :math:`f(\\mathbf{x}) = 10N \sum_{i=1}^N x_i^2 - 10 \\cos(2\\pi x_i)`
+         - :math:`f(\\mathbf{x}) = 10N + \sum_{i=1}^N x_i^2 - 10 \\cos(2\\pi x_i)`
 
     .. plot:: code/benchmarks/rastrigin.py
        :width: 67 %
@@ -343,8 +343,6 @@ def shekel(individual, a, c):
     of maxima is given by the length of any of the arguments *a* or *c*, *a*
     is a matrix of size :math:`M\\times N`, where *M* is the number of maxima
     and *N* the number of dimensions and *c* is a :math:`M\\times 1` vector.
-    The matrix :math:`\\mathcal{A}` can be seen as the position of the maxima
-    and the vector :math:`\\mathbf{c}`, the width of the maxima.
     
     :math:`f_\\text{Shekel}(\mathbf{x}) = \\sum_{i = 1}^{M} \\frac{1}{c_{i} + 
     \\sum_{j = 1}^{N} (x_{j} - a_{ij})^2 }`
@@ -360,7 +358,7 @@ def shekel(individual, a, c):
     .. plot:: code/benchmarks/shekel.py
         :width: 67 %
     """
-    return sum((1. / (c[i] + sum((x - a[i][j])**2 for j, x in enumerate(individual)))) for i in range(len(c))),
+    return sum((1. / (c[i] + sum((individual[j] - aij)**2 for j, aij in enumerate(a[i])))) for i in range(len(c))),
 
 # Multiobjectives
 def kursawe(individual):
@@ -467,7 +465,7 @@ def zdt6(individual):
     return f1, f2
 
 def dtlz1(individual, obj):
-    """DTLZ1 mutliobjective function. It returns a tuple of *obj* values. 
+    """DTLZ1 multiobjective function. It returns a tuple of *obj* values. 
     The individual must have at least *obj* elements.
     From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective 
     Optimization Test Problems. CEC 2002, p. 825 - 830, IEEE Press, 2002.
@@ -495,7 +493,7 @@ def dtlz1(individual, obj):
     return f
 
 def dtlz2(individual, obj):
-    """DTLZ2 mutliobjective function. It returns a tuple of *obj* values. 
+    """DTLZ2 multiobjective function. It returns a tuple of *obj* values. 
     The individual must have at least *obj* elements.
     From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective 
     Optimization Test Problems. CEC 2002, p. 825 - 830, IEEE Press, 2002.
@@ -518,11 +516,12 @@ def dtlz2(individual, obj):
     xm = individual[obj-1:]
     g = sum((xi-0.5)**2 for xi in xm)
     f = [(1.0+g) *  reduce(mul, (cos(0.5*xi*pi) for xi in xc), 1.0)]
-    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi*pi) for xi in xc[:m-1]), 1) * sin(0.5*xc[m]*pi) for m in reversed(xrange(obj-1)))
+    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi*pi) for xi in xc[:m]), 1) * sin(0.5*xc[m]*pi) for m in range(obj-2, -1, -1))
+
     return f
 
 def dtlz3(individual, obj):
-    """DTLZ3 mutliobjective function. It returns a tuple of *obj* values. 
+    """DTLZ3 multiobjective function. It returns a tuple of *obj* values. 
     The individual must have at least *obj* elements.
     From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective 
     Optimization Test Problems. CEC 2002, p. 825 - 830, IEEE Press, 2002.
@@ -545,11 +544,11 @@ def dtlz3(individual, obj):
     xm = individual[obj-1:]
     g = 100 * (len(xm) + sum((xi-0.5)**2 - cos(20*pi*(xi-0.5)) for xi in xm))
     f = [(1.0+g) *  reduce(mul, (cos(0.5*xi*pi) for xi in xc), 1.0)]
-    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi*pi) for xi in xc[:m-1]), 1) * sin(0.5*xc[m]*pi) for m in reversed(xrange(obj-1)))
+    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi*pi) for xi in xc[:m]), 1) * sin(0.5*xc[m]*pi) for m in range(obj-2, -1, -1))
     return f
 
 def dtlz4(individual, obj, alpha):
-    """DTLZ4 mutliobjective function. It returns a tuple of *obj* values. The
+    """DTLZ4 multiobjective function. It returns a tuple of *obj* values. The
     individual must have at least *obj* elements. The *alpha* parameter allows
     for a meta-variable mapping in :func:`dtlz2` :math:`x_i \\rightarrow
     x_i^\\alpha`, the authors suggest :math:`\\alpha = 100`.
@@ -574,9 +573,60 @@ def dtlz4(individual, obj, alpha):
     xm = individual[obj-1:]
     g = sum((xi-0.5)**2 for xi in xm)
     f = [(1.0+g) *  reduce(mul, (cos(0.5*xi**alpha*pi) for xi in xc), 1.0)]
-    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi**alpha*pi) for xi in xc[:m-1]), 1) * sin(0.5*xc[m]**alpha*pi) for m in reversed(xrange(obj-1)))
+    f.extend((1.0+g) * reduce(mul, (cos(0.5*xi**alpha*pi) for xi in xc[:m]), 1) * sin(0.5*xc[m]**alpha*pi) for m in range(obj-2, -1, -1))
     return f
 
+def dtlz5(ind, n_objs):
+    """DTLZ5 multiobjective function. It returns a tuple of *obj* values. The
+    individual must have at least *obj* elements.
+    From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective
+    Optimization Test Problems. CEC 2002, p. 825-830, IEEE Press, 2002.
+    """
+    g = lambda x: sum([(a - 0.5)**2 for a in x])
+    gval = g(ind[n_objs-1:])
+    
+    theta = lambda x: pi / (4.0 * (1 + gval)) * (1 + 2 * gval * x)
+    fit = [(1 + gval) * cos(pi / 2.0 * ind[0]) * reduce(lambda x,y: x*y, [cos(theta(a)) for a in ind[1:]])]
+           
+    for m in reversed(range(1, n_objs)):
+        if m == 1:
+            fit.append((1 + gval) * sin(pi / 2.0 * ind[0]))
+        else:
+            fit.append((1 + gval) * cos(pi / 2.0 * ind[0]) *
+                       reduce(lambda x,y: x*y, [cos(theta(a)) for a in ind[1:m-1]], 1) * sin(theta(ind[m-1])))
+    return fit
+
+def dtlz6(ind, n_objs):
+    """DTLZ6 multiobjective function. It returns a tuple of *obj* values. The
+    individual must have at least *obj* elements.
+    From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective
+    Optimization Test Problems. CEC 2002, p. 825-830, IEEE Press, 2002.
+    """
+    gval = sum([a**0.1 for a in ind[n_objs-1:]])
+    theta = lambda x: pi / (4.0 * (1 + gval)) * (1 + 2 * gval * x)
+    
+    fit = [(1 + gval) * cos(pi / 2.0 * ind[0]) *
+           reduce(lambda x,y: x*y, [cos(theta(a)) for a in ind[1:]])]
+
+    for m in reversed(range(1, n_objs)):
+        if m == 1:
+            fit.append((1 + gval) * sin(pi / 2.0 * ind[0]))
+        else:
+            fit.append((1 + gval) * cos(pi / 2.0 * ind[0]) *
+                       reduce(lambda x,y: x*y, [cos(theta(a)) for a in ind[1:m-1]], 1) * sin(theta(ind[m-1])))
+    return fit
+
+def dtlz7(ind, n_objs):
+    """DTLZ7 multiobjective function. It returns a tuple of *obj* values. The
+    individual must have at least *obj* elements.
+    From: K. Deb, L. Thiele, M. Laumanns and E. Zitzler. Scalable Multi-Objective
+    Optimization Test Problems. CEC 2002, p. 825-830, IEEE Press, 2002.
+    """
+    gval = 1 + 9.0 / len(ind[n_objs-1:]) * sum([a for a in ind[n_objs-1:]])
+    fit = [ind for ind in ind[:n_objs-1]]
+    fit.append((1 + gval) * (n_objs - sum([a / (1.0 + gval) * (1 + sin(3 * pi * a)) for a in ind[:n_objs-1]])))
+    return fit
+
 def fonseca(individual):
     """Fonseca and Fleming's multiobjective function.
     From: C. M. Fonseca and P. J. Fleming, "Multiobjective optimization and
@@ -617,3 +667,22 @@ def poloni(individual):
     B_2 = 1.5 * sin(x_1) - cos(x_1) + 2 * sin(x_2) - 0.5 * cos(x_2)
     return 1 + (A_1 - B_1)**2 + (A_2 - B_2)**2, (x_1 + 3)**2 + (x_2 + 1)**2
 
+def dent(individual, lambda_ = 0.85):
+    """Test problem Dent. Two-objective problem with a "dent". *individual* has
+    two attributes that take values in [-1.5, 1.5].
+    From: Schuetze, O., Laumanns, M., Tantar, E., Coello Coello, C.A., & Talbi, E.-G. (2010).
+    Computing gap free Pareto front approximations with stochastic search algorithms.
+    Evolutionary Computation, 18(1), 65--96. doi:10.1162/evco.2010.18.1.18103
+
+    Note that in that paper Dent source is stated as:
+    K. Witting and M. Hessel von Molo. Private communication, 2006.
+    """
+    d = lambda_ * exp(-(individual[0] - individual[1]) ** 2)
+    f1 = 0.5 * (sqrt(1 + (individual[0] + individual[1]) ** 2) + \
+                sqrt(1 + (individual[0] - individual[1]) ** 2) + \
+                individual[0] - individual[1]) + d
+    f2 = 0.5 * (sqrt(1 + (individual[0] + individual[1]) ** 2) + \
+                sqrt(1 + (individual[0] - individual[1]) ** 2) - \
+                individual[0] + individual[1]) + d
+    return f1, f2
+
