@@ -11,24 +11,32 @@ from collections import defaultdict
 # Non-Dominated Sorting   (NSGA-II)  #
 ######################################
 
-def selNSGA2(individuals, k):
+def selNSGA2(individuals, k, nd='standard'):
     """Apply NSGA-II selection operator on the *individuals*. Usually, the
     size of *individuals* will be larger than *k* because any individual
     present in *individuals* will appear in the returned list at most once.
     Having the size of *individuals* equals to *k* will have no effect other
-    than sorting the population according according to their front rank. The
+    than sorting the population according to their front rank. The
     list returned contains references to the input *individuals*. For more
     details on the NSGA-II operator see [Deb2002]_.
     
     :param individuals: A list of individuals to select from.
     :param k: The number of individuals to select.
+    :param nd: Specify the non-dominated algorithm to use: 'standard' or 'log'.
     :returns: A list of selected individuals.
     
     .. [Deb2002] Deb, Pratab, Agarwal, and Meyarivan, "A fast elitist
        non-dominated sorting genetic algorithm for multi-objective
        optimization: NSGA-II", 2002.
     """
-    pareto_fronts = sortNondominated(individuals, k)
+    if nd == 'standard':
+        pareto_fronts = sortNondominated(individuals, k)
+    elif nd == 'log':
+        pareto_fronts = sortLogNondominated(individuals, k)
+    else:
+        raise Exception('selNSGA2: The choice of non-dominated sorting '
+                        'method "{0}" is invalid.'.format(nd))
+
     for front in pareto_fronts:
         assignCrowdingDist(front)
     
@@ -212,7 +220,7 @@ def median(seq, key=identity):
     else:
         return (key(sseq[(length - 1) // 2]) + key(sseq[length // 2])) / 2.0
 
-def sortLogNondominated(individuals):
+def sortLogNondominated(individuals, k, first_front_only=False):
     """Sort *individuals* in pareto non-dominated fronts using the Generalized
     Reduced Run-Time Complexity Non-Dominated Sorting Algorithm presented by
     Fortin et al. (2013).
@@ -221,6 +229,9 @@ def sortLogNondominated(individuals):
     :returns: A list of Pareto fronts (lists), with the first list being the
               true Pareto front.
     """
+    if k == 0:
+        return []
+    
     #Separate individuals according to unique fitnesses
     unique_fits = defaultdict(list)
     for i, ind in enumerate(individuals):
@@ -241,7 +252,17 @@ def sortLogNondominated(individuals):
     for fit in fitnesses:
         index = front[fit]
         pareto_fronts[index].extend(unique_fits[fit])
-    return pareto_fronts
+
+    # Keep only the fronts required to have k individuals.
+    if not first_front_only:
+        count = 0
+        for i, front in enumerate(pareto_fronts):
+            count += len(front)
+            if count >= k:
+                return pareto_fronts[:i+1]
+        return pareto_fronts
+    else:
+        return pareto_fronts[0]
 
 def sortNDHelperA(fitnesses, obj, front):
     """Create a non-dominated sorting of S on the first M objectives"""
